<?php

namespace Drupal\activitypub\Plugin\activitypub\type;

use Drupal\activitypub\Entity\ActivityPubActivityInterface;
use Drupal\activitypub\Services\Type\TypePluginBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * The ActivityPub context.
 *
 * @ActivityPubType(
 *   id = "activitypub_context",
 *   label = @Translation("Context"),
 * )
 */
class Context extends TypePluginBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function onActivityPostSave(ActivityPubActivityInterface $activity, $update = TRUE) {
    if (in_array($activity->getType(), $this->activityPubUtility->getTimelineTypes()) && !$update) {
      // Is the context config entity enabled or not.
      /** @var \Drupal\activitypub\Entity\ActivityPubTypeInterface $activityPubType */
      $activityPubType = $this->entityTypeManager->getStorage('activitypub_type')->load('context');
      if ($activityPubType && $activityPubType->isEnabled()) {
        $payload = @json_decode($activity->getPayLoad(), TRUE);
        if ((isset($payload['object']) && isset($payload['object']['inReplyTo']) && !empty($payload['object']['inReplyTo'])) || (in_array($activity->getType(), ['Like', 'Announce']))) {

          $values = [
            'uid' => $activity->getOwnerId(),
            'collection' => ActivityPubActivityInterface::CONTEXT,
            'config_id' => 'context',
            'type' => '',
            'actor' => $activity->getObject(),
            'object' => $activity->getActor(),
            'external_id' => $activity->id(),
            'status' => 0,
            'payload' => $activity->getPayLoad(),
          ];

          /** @var \Drupal\activitypub\Entity\ActivityPubActivityInterface $contextActivity */
          $contextActivity = $this->entityTypeManager->getStorage('activitypub_activity')->create($values);
          $contextActivity->save();
          $this->activityPubProcessClient->createQueueItem($contextActivity);
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function doInboxProcess(ActivityPubActivityInterface $activity, EntityInterface $entity = NULL) {
    if (!$activity->isProcessed() && $activity->getConfigID() == 'context') {
      $url = NULL;

      /** @var \Drupal\activitypub\Entity\ActivityPubActivityInterface $loadActivity */
      $loadActivity = $this->entityTypeManager->getStorage('activitypub_activity')->load($activity->getExternalId());

      $payload = @json_decode($loadActivity->getPayLoad(), TRUE);
      if (isset($payload['object']) && isset($payload['object']['inReplyTo']) && !empty($payload['object']['inReplyTo'])) {
        $url = $payload['object']['inReplyTo'];
      }
      elseif (in_array($loadActivity->getType(), ['Like', 'Announce'])) {
        $url = $loadActivity->getObject();
      }

      if (isset($url)) {
        $client = \Drupal::httpClient();
        try {
          $options = [
            'headers' => [
              'Accept' => 'application/activity+json',
            ],
          ];
          $response = $client->get($url, $options);
          $body = $response->getBody()->getContents();
          $context = @json_decode($body, TRUE);
          if (isset($context['id']) && $context['id'] == $url) {
            $loadActivity->set('context', $body)->save();
          }
        }
        catch (\Exception $ignored) {
          $this->logger->warning('Error fetching context for activity @id: @message', ['@id' => $activity->id(), '@message' => $ignored->getMessage()]);
        }
      }
    }
  }

}
