<?php

namespace Drupal\activitypub\Plugin\activitypub\type;

use Drupal\activitypub\Entity\ActivityPubActivityInterface;
use Drupal\activitypub\Entity\ActivityPubTypeInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * The ActivityPub Direct Message types.
 *
 * @ActivityPubType(
 *   id = "activitypub_direct_message_types",
 *   label = @Translation("Direct message"),
 *   weight = 10
 * )
 */
class DirectMessageTypes extends DynamicTypes {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['info'] = [
      '#markup' => '<p><strong>Experimental</strong>, use this plugin with care.</p>',
      '#weight' => -100,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function onActivityInboxPreSave(ActivityPubActivityInterface $activity, &$doSave) {
    $payload = @json_decode($activity->getPayLoad(), TRUE);
    if ($activity->getType() == 'Create' && empty($payload['cc']) && !in_array(ActivityPubActivityInterface::PUBLIC_URL, $payload['to'])) {
      $activity->set('direct', TRUE);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onActivityOutboxPreSave(ActivityPubActivityInterface $activity, EntityInterface $entity, ActivityPubTypeInterface $activityPubType, &$doSave) {
    parent::onActivityOutboxPreSave($activity, $entity, $activityPubType, $doSave);
    if ($activityPubType->getPlugin()['id'] == 'activitypub_direct_message_types') {
      $doSave = TRUE;
      $activity->set('direct', TRUE);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build(ActivityPubActivityInterface $activity, EntityInterface $entity = NULL) {
    $return = parent::build($activity, $entity);
    $to = $activity->getTo();
    $return['to'] = $to;
    if (isset($return['object']['to'])) {
      $return['object']['to'] = $to;
    }
    $return['cc'] = [];
    if (isset($return['object']['cc'])) {
      $return['object']['cc'] = [];
    }
    $return['_drupal_internal_direct'] = TRUE;

    // Add mention tag.
    $mention = [];
    if (!empty($to[0])) {
      $href = $to[0];
      $parsed = parse_url($href);
      if (!empty($parsed['host']) && !empty($parsed['path'])) {
        // A bit naive, but let's fix this later.
        $ex = explode('/', $parsed['path']);
        $username = array_pop($ex);
        $name = '@' . $username . '@' . $parsed['host'];
      }
      else {
        $name = $href;
      }
      $mention = [
        'type' => 'Mention',
        'href' => $href,
        'name' => $name,
      ];
    }
    if (!empty($mention)) {
      $return['object']['tag'] = [ (object) $mention];
    }

    return $return;
  }

}
