<?php

namespace Drupal\Tests\activitypub\Functional;

use Drupal\Core\Url;

/**
 * Tests Like functionality.
 *
 * @group activitypub
 */
class LikeTest extends ActivityPubTestBase {

  /**
   * Tests like
   */
  public function testLike() {
    $assert_session = $this->assertSession();

    /** @var \Drupal\activitypub\Entity\Storage\ActivityPubActorStorageInterface $storage */
    $storage = \Drupal::entityTypeManager()->getStorage('activitypub_activity');

    // Setup ActivityPub actor.
    $this->enableActivityPub($assert_session);
    $this->drupalLogout();

    // Set outbox handler.
    $this->setOutboxHandler();

    $actor_href = Url::fromRoute('activitypub.user.self', ['user' => $this->authenticatedUserOne->id(), 'activitypub_actor' => $this->accountNameOne], ['absolute' => TRUE])->toString();

    // Create type.
    $this->createType('Like', 'page', '');
    // Enable context.
    $this->setTypeStatus('context');

    // Send to outbox.
    $this->drupalLogin($this->authenticatedUserOne);
    $this->drupalGet('node/add/page');
    $edit = [
      'title[0][value]' => 'Hello like!',
      'activitypub_type' => 'map',
      'activitypub_to' => $actor_href,
      'status[value]' => TRUE,
    ];
    $this->submitForm($edit, 'Save');
    $this->drupalLogout();

    $count = \Drupal::queue(ACTIVITYPUB_OUTBOX_QUEUE)->numberOfItems();
    self::assertEquals(1, $count);
    $count = \Drupal::queue(ACTIVITYPUB_INBOX_QUEUE)->numberOfItems();
    self::assertEquals(1, $count);
    $items = $this->getQueuedItems();
    $delta = 0;
    foreach ($items as $item) {
      if ($delta === 0) {
        self::assertEquals(ACTIVITYPUB_OUTBOX_QUEUE, $item->name);
      }
      else {
        self::assertEquals(ACTIVITYPUB_INBOX_QUEUE, $item->name);
      }
      $delta++;
    }

    /** @var \Drupal\activitypub\Entity\ActivityPubActivityInterface $activity */
    $activity = $storage->load(1);
    self::assertEquals('map', $activity->getConfigID());
    self::assertEquals('outbox', $activity->getCollection());
    /** @var \Drupal\activitypub\Entity\ActivityPubActivityInterface $activity2 */
    $activity2 = $storage->load(2);
    self::assertEquals('context', $activity2->getConfigID());
    self::assertEquals('context', $activity2->getCollection());
    self::assertEquals(1, $activity2->getExternalId());
  }

}
